﻿using System;
using System.Collections.Generic;
using UnityEngine;
using System.Threading;


public class Main : MonoBehaviour
{
    public GameObject car;
    private float carX, carY, carZ, carAngle;
    public GameObject servo;
    public ParticleSystem pointsSystem;
    private ParticleSystem.Particle[] pointsArr;
    public Color pointColor;
    public float pointSize = 100f;
    public int maxNumberPoints = 10;
    private int pointIndex=0;

    public LineRenderer carLine;

    private UInt16 servoAngle = 0;

    Thread serialThread;
    private object pointListQueueLock = new object();
    Queue<List<LidarPoint>> lidarPointListQueue = new Queue<List<LidarPoint>>();


    void Start()
    {
        pointsArr = new ParticleSystem.Particle[maxNumberPoints];
        pointsSystem.Emit(maxNumberPoints);
        pointsSystem.GetParticles(pointsArr);

        carLine.positionCount = 1;
        carLine.SetPosition(carLine.positionCount - 1, new Vector3(0, 0, 0));

        XbeeCommunication.inicialize();
        XbeeCommunication.open();

        setCarTransformProperties();
        
    }



    void Update()
    {
        lock (pointListQueueLock)
        {
            if (lidarPointListQueue.Count > 0)
            {
                addPointsToParticleSystem(lidarPointListQueue.Dequeue());   
            }
        }
        pointsSystem.SetParticles(pointsArr, pointIndex);

    }

    public void LateUpdate()
    {
        servo.transform.rotation = Quaternion.Euler(0, car.transform.rotation.eulerAngles.y+90 + servoAngle, 0);
    }

    //******************* Inicio funciones referidas al escaneo ****************************************

    //Metodo que recibe los datos desde Xbee (Ejecuta hilo aparte)
    void receiveMappingData()
    {
        int id = 1;
        DateTime measureDateTime;
        LidarMeasure[] lidarMeasures;
        List<LidarPoint> lidarPoints;
        XbeeFrame RXframe;

        while ( Mapping.getState()!="off" && pointCloudFile.exists() && XbeeCommunication.isOpen() )
        {
            //Recibe trama
            RXframe = XbeeCommunication.receiveFrame();
            if (RXframe != null)
            {
                //Extrer y convertir medidas
                lidarMeasures = LidarMeasure.extractMeasuresFromFrame(RXframe.getFramePayload(), servoAngle);
                measureDateTime = DateTime.Now;
                lidarPoints = LidarMeasure.transformMeasuresToPoints(lidarMeasures, carX, carY, carZ, carAngle);

                foreach (LidarPoint point in lidarPoints)
                {
                    //Alamcena en archico .txt
                    if( pointCloudFile.saveData(id, point, measureDateTime) )
                    {
                        id++;
                        if (id % 500 == 0)
                        {
                            //Envia comando para mover servo
                            servoAngle++;                       
                            MoveServo.toAngle(servoAngle);
                            Debug.Log("Servo Angle");
                        }
                        Debug.Log("Punto#");
                    }
                }
                lock (pointListQueueLock)
                {
                    lidarPointListQueue.Enqueue(lidarPoints);
                }
                Thread.Sleep(9);
            }
            if(servoAngle >= 180)
            {
                endingMapping();
            }
            while (Mapping.getState() == "stopped"){}
        }
    }

    /*Agrega puntos al sistema de particulas
     * @param lidarPoints lista de puntos
     */
    private void addPointsToParticleSystem(List<LidarPoint> lidarPoints)
    {
        foreach (LidarPoint point in lidarPoints)
        {
            pointsArr[pointIndex].position = point.XYZposition;
            pointsArr[pointIndex].startSize = pointSize;
            pointIndex++;
        }
    }
    //******************* Fin funciones referidas al escaneo ****************************************




    //******************* Inicio funciones referidas al avance del vehiculo ****************************************
    /*Recibe comando de interfaz de usuario y envia comando
     * de avance de vehiculo a arduino y lo mueve en Unity
     * @param moveFwd cantidad de movimiento en cm
     */
    public void movingForward(String moveFwd)
    {
        try
        {
            MoveVehicle.forward(UInt16.Parse(moveFwd));
            car.transform.position = car.transform.position + car.transform.forward * Int32.Parse(moveFwd) * 10;
            servo.transform.position = servo.transform.position + car.transform.forward * Int32.Parse(moveFwd) * 10;
            setCarTransformProperties();
        }
        catch
        {}
    }

    /*Recibe comando de interfaz de usuario y envia comando
     * de retroceso de vehiculo a arduino y lo mueve en Unity
     * @param moveBwd cantidad de movimiento en cm
     */
    public void movingBackward(String moveBwd)
    {
        try 
        {
            MoveVehicle.backward(UInt16.Parse(moveBwd));
            car.transform.position = car.transform.position + car.transform.forward * Int32.Parse(moveBwd) * -10;
            servo.transform.position = servo.transform.position + car.transform.forward * Int32.Parse(moveBwd) * -10;
            setCarTransformProperties();
        }
        catch 
        { }
    }

    /*Recibe comando de interfaz de usuario y envia comando
     * de giro izquierda a arduino y lo mueve en Unity
     * @param leftRot cantidad de movimiento en cm
     */
    public void rotateLeft(String leftRot)
    {
        try {
            MoveVehicle.toTheLeft(UInt16.Parse(leftRot));
            car.transform.rotation = Quaternion.Euler(0, car.transform.rotation.eulerAngles.y - Int32.Parse(leftRot), 0);
            servo.transform.rotation = Quaternion.Euler(0, servo.transform.rotation.eulerAngles.y - Int32.Parse(leftRot), 0);
            setCarTransformProperties();

            Debug.Log("rotacion =" + car.transform.rotation.eulerAngles);
     
        }
        catch
        { }
    }

    /*Recibe comando de interfaz de usuario y envia comando
     * de giro derecha a arduino y lo mueve en Unity
     * @param rightRot cantidad de movimiento en cm
     */
    public void rotateRight(String rightRot)
    {
        try
        {
            MoveVehicle.toTheRight(UInt16.Parse(rightRot));
            car.transform.rotation = Quaternion.Euler(0, car.transform.rotation.eulerAngles.y - Int32.Parse(rightRot) * -1, 0);
            servo.transform.rotation = Quaternion.Euler(0, servo.transform.rotation.eulerAngles.y - Int32.Parse(rightRot) * -1, 0);
            setCarTransformProperties();
        }
        catch
        { }
    }
    //******************* Fin funciones referidas al avance del vehiculo ****************************************



    //******************* Inicio funciones de los botones ****************************************
    public void startMapping()
    {
        if (Mapping.getState() == "off")
        {
            if (Mapping.on())
            {
                MoveServo.inicialize();
                pointCloudFile.inicialize();
                serialThread = new Thread(receiveMappingData);
                serialThread.IsBackground = true;
                serialThread.Start();
            }
            else
            {
                XbeeCommunication.dispose();
            }

        }
        if (Mapping.getState() == "stopped")
        {
            Mapping.on();
        }
    }
    public void stopMapping()
    {
        Mapping.stop();
    }
    public void deletePoints()
    {
        Debug.Log("Borrando puntos");
        pointsSystem.Clear();
        pointIndex = 0;
    }

    public void finishMapping()
    {
        Mapping.stop();
        endingMapping();
    }
    public void finishApp()
    {
        Mapping.stop();
        endingMapping();
        XbeeCommunication.dispose();
        Application.Quit();
    }

    private void endingMapping()
    {
        if (Mapping.off())
        {
            servoAngle = 0;
            MoveServo.toAngle(servoAngle);
            if (pointCloudFile.exists())
            {
                pointCloudFile.dispose();
            }
            pointsSystem.Pause();
        }
    }

    //******************* Fin funciones de los botones ****************************************

    //Dibuja la linea de desplazamiento del vehiculo
    private void setCarTransformProperties()
    {
        carX = car.transform.position.x;
        carY = car.transform.position.y;
        carZ = car.transform.position.z;
        carAngle = car.transform.rotation.eulerAngles.y;
        carLine.positionCount++;
        carLine.SetPosition(carLine.positionCount - 1, new Vector3(carX, carY, carZ));
    }



}


