﻿using System;
using System.IO.Ports;
using UnityEngine;

//Clase que se encarga de la comunicacion serial con el modulo Xbee
public static class XbeeCommunication
{
    private static readonly byte startFrameByte = 0x7E;
    private static readonly byte RXframeType = 0x90;
    public static readonly int amountRXdata = 14;
    public static readonly int amountTXdata = 1;

    private static SerialPort serialXbee;
    private static XbeeFrame RXframe;
    private static XbeeFrame TXframe;
    private static byte[] serialBuffer;

    public static void inicialize()
    {
        //serialXbee = new SerialPort("COM3", 115200, Parity.None, 8, StopBits.One); //PC
        serialXbee = new SerialPort("COM5", 115200, Parity.None, 8, StopBits.One); //Portatil
        serialXbee.ReadTimeout = 500;
        serialXbee.WriteTimeout = 500;
        RXframe = new XbeeFrame("RX_FRAME");
        TXframe = new XbeeFrame("TX_FRAME");
        serialBuffer = new byte[RXframe.FRAME_TOTAL_BYTES];
    }

    public static bool open()
    {
        try
        {
            serialXbee.Open();
            return true;
        }
        catch
        {
            return false;
        }
    }


    public static void dispose()
    {
        serialXbee.Dispose();
    }
    public static void close()
    {
        serialXbee.Close();
    }
    public static bool isOpen()
    {
        return serialXbee.IsOpen;
    }


    public static XbeeFrame receiveFrame()
    {
        serialBuffer[0] = 0x00;
        bool rxOk = false;
        try
        {
            while (!rxOk)
            {
                serialXbee.Read(serialBuffer, 0, 1);
                if (serialBuffer[0] == startFrameByte)
                {
                    serialXbee.Read(serialBuffer, 1, 3);
                    if (serialBuffer[3] == RXframeType)
                        rxOk = true;
                }
            }
            serialXbee.Read(serialBuffer, 4, RXframe.FRAME_TOTAL_BYTES - 4);
            RXframe.setframe(serialBuffer);
            Array.Clear(serialBuffer, 0, serialBuffer.Length);
            RXframe.disassembleFrame();
            return RXframe;
        }
        catch (Exception e)
        {
            Debug.LogError("Data receive ERROR: " + e.Message);
            return null;
        }
        
    }

    public static XbeeFrame createFrame(char letter, UInt16 value)
    {
        TXframe.getFramePayload().SetValue((byte)letter, 0);
        TXframe.getFramePayload().SetValue((byte)(value >> 8), 1);
        TXframe.getFramePayload().SetValue((byte)value, 2);
        return TXframe;
    }
    public static bool sendData(XbeeFrame frame)
    {
        try 
        {
            serialXbee.Write(frame.getFrame(), 0, frame.FRAME_TOTAL_BYTES);
            return true;
        }
        catch
        {
            return false;
        }
        
    }



   


}
