﻿
//Clase que gestiona las tramas API
public class XbeeFrame
{
    public int FRAME_TOTAL_BYTES { get; }
    public int FRAME_HEADER_BYTES { get; }
    public int FRAME_PAYLOAD_BYTES { get; }

    private byte[] frameHeader;
    private byte[] framePayload;
    private byte[] frame;

    private int chksum;

    //De acuerdo al tipo establece la cabecera de la trama
    public XbeeFrame(string frameType)
    {
        if (frameType == "RX_FRAME")
        {
            FRAME_HEADER_BYTES = 15;
            FRAME_PAYLOAD_BYTES = XbeeCommunication.amountRXdata * LidarMeasure.measureByteSize; //medidas de 8 bytes cada una
            FRAME_TOTAL_BYTES = FRAME_HEADER_BYTES + FRAME_PAYLOAD_BYTES + 1;

            frame = new byte[FRAME_TOTAL_BYTES];
            frameHeader = new byte[FRAME_HEADER_BYTES];
            framePayload = new byte[FRAME_PAYLOAD_BYTES];
        }
        if (frameType == "TX_FRAME")
        {
            FRAME_HEADER_BYTES = 17;
            FRAME_PAYLOAD_BYTES = XbeeCommunication.amountTXdata * Command.commandByteSize; //comando de 3 bytes cada una, 1 char y un int16
            FRAME_TOTAL_BYTES = FRAME_HEADER_BYTES + FRAME_PAYLOAD_BYTES + 1; 

            frame = new byte[FRAME_TOTAL_BYTES];
            framePayload = new byte[FRAME_PAYLOAD_BYTES];
            frameHeader = new byte[] { 0x7E, 0x00, 0x11, 0x10, 0x01, 0x00, 0x13, 0xA2, 0x00, 0x41, 0xBB, 0xC1, 0xA9, 0xFF, 0xFE, 0x00, 0x00 };
        }
        this.chksum = 0;
    }

    public byte[] getFrameHeader()
    {
        return this.frameHeader;
    }
    public bool setFrameHeader(byte[] frameHeader)
    {
        if (frameHeader != null)
        {
            this.frameHeader = frameHeader;
            return true;
        }
        return false;
    }
    public byte[] getFramePayload()
    {
        return this.framePayload;
    }
    public bool setFramePayload(byte[] framePayload)
    {  
        if (framePayload != null)
        {
            this.framePayload = framePayload;
            return true;
        }
        return false;
    }
    public byte[] getFrame()
    {
        return assembleFrame();
    }
    public void setframe(byte[] frame)
    {   
        for(int i=0; i< FRAME_TOTAL_BYTES; i++)
        {
            this.frame[i] = frame[i];
        }        
    }
     //Ensambla una trama aprtir de un header y payload
    private byte[] assembleFrame()
    {
        frameHeader.CopyTo(frame, 0);
        framePayload.CopyTo(frame, FRAME_HEADER_BYTES);
        frame[FRAME_TOTAL_BYTES - 1] = calculateChksum();
        return frame;
    }

    //Desensambla una trama en header, payload, checksum
    public void disassembleFrame()
    {
        for (int i = 0; i < FRAME_TOTAL_BYTES - 1; i++)
        {
            if (i < FRAME_HEADER_BYTES)
                frameHeader[i] = frame[i];
            else
                framePayload[i- FRAME_HEADER_BYTES] = frame[i];
        }
    }

    //Calcula el checksum de una trama
    public byte calculateChksum()
    {
        chksum = 0;
        for (int i = 3; i < FRAME_HEADER_BYTES; i++)
        {
            chksum += frameHeader[i];
        }

        for (int i = 0; i < FRAME_PAYLOAD_BYTES; i++)
        {
            chksum += framePayload[i];
        }
        chksum = 0xFF - (0xFF & chksum);
        return (byte)chksum;
    }

    //Verifica el checksum de una trama
    public bool frameOk()
    {
        if (calculateChksum() == frame[FRAME_TOTAL_BYTES - 1])
            return true;
        return false;
    }

}
