
// Free-RTOS includes
#include <Arduino_FreeRTOS.h>
#include "FreeRTOSConfig.h"
#include <task.h>
#include <semphr.h>
#include "lidarMeasure.h"
#include "SPIcommunication.h"
#include "motorControl.h"

//Definicion de prioridades tareas freeRTOS
#define RECEIVE_LIDAR_TASK_PRIORITY           ( tskIDLE_PRIORITY + 1 )
#define SERVO_TASK_PRIORITY                   ( tskIDLE_PRIORITY + 1 )
#define SEND_SPI_TASK_PRIORITY                ( tskIDLE_PRIORITY + 1 )
#define RECEIVE_SPI_TASK_PRIORITY             ( tskIDLE_PRIORITY + 1 )
#define MOTOR_TASK_PRIORITY                   ( tskIDLE_PRIORITY + 1 )
#define BLINK_TASK_PRIORITY                   ( tskIDLE_PRIORITY + 1 )

//Definicion de tamño de memoria tareas freeRTOS
#define RECEIVE_LIDAR_TASK_STACK_SIZE         ( 100 )  //100
#define SERVO_TASK_STACK_SIZE                 ( 60 )  //50
#define SEND_SPI_TASK_STACK_SIZE              ( 200 ) //165-para 10 medidas float
#define RECEIVE_SPI_TASK_STACK_SIZE           ( 80 )  //70
#define MOTOR_TASK_STACK_SIZE                 ( 60 ) //60
#define BLINK_TASK_STACK_SIZE                 ( 45 )  //45

//Definicion de pines arduino
#define LED_BLINK                             2
#define SERVO_CONTROL                         19

//Semaforos
static SemaphoreHandle_t SPIMutex;

//Colas
QueueHandle_t lidarSPIQueue;

//Tareas FreeRTOS
//Tarea que recibe las medidas del lidar
static void lidarTask(void *pvParameters);
//Control del servo
static void servoTask(void *pvParameters);
//Tarea para enviar tramas API via SPI hacia Xbee
static void sendSPITask(void *pvParameters);
//Tarea que recibe tramas API via SPI desde Xbee
static void receiveSPITask(void *pvParameters);
//Control del movimiento del vehiculo
static void motorTask(void *pvParameters);
static void blinkTask(void *pvParameters);

//Funciones
//Funcion que interpreta comando de Unity
void applyCommand();
/*Función que controla el ancho de pulso
 *@param servoAngle angulo donde se debe ubicar el servo
*/
void moveServo(unsigned int servoAngle);
void pinConfig();
void led_arduino_on();
