#include "Vehiculo.h"

Measure_t measure;
RPLidar lidar;
bool startMapping = false;
bool startMotors = false;
bool startServo = false;
UnityCommand_t unityCommand;

void setup()
{
  pinConfig();
  lidar.begin(Serial);
  SPI.begin();

  bool error = false;
  //Inicializacion semaforo
  SPIMutex = xSemaphoreCreateMutex();
  if (SPIMutex == NULL) error = true;

  //Inicializacion cola
  lidarSPIQueue = xQueueCreate( 1, sizeof( Measure_t ));
  if ((lidarSPIQueue == NULL) ) error = true;

  //Inicializacion de tareas ------------------------------------------------------------------
  BaseType_t retVal;
  retVal = xTaskCreate(receiveLidarTask, "ReceiveLidarTask", RECEIVE_LIDAR_TASK_STACK_SIZE,
                      NULL, RECEIVE_LIDAR_TASK_PRIORITY, NULL );
  if (retVal < 0) error = true;
  retVal = xTaskCreate(servoTask, "servoTask", SERVO_TASK_STACK_SIZE,
                      NULL, SERVO_TASK_PRIORITY, NULL );
  if (retVal < 0) error = true;
  retVal = xTaskCreate(sendSPITask, "sendSPITask", SEND_SPI_TASK_STACK_SIZE,
                      NULL, SEND_SPI_TASK_PRIORITY, NULL );
  if (retVal < 0) error = true;
  retVal = xTaskCreate(receiveSPITask, "ReceiveSPITask", RECEIVE_SPI_TASK_STACK_SIZE,
                      NULL, RECEIVE_SPI_TASK_PRIORITY, NULL );
  if (retVal < 0) error = true;
  retVal = xTaskCreate(motorTask, "motorTask", MOTOR_TASK_STACK_SIZE,
                      NULL, MOTOR_TASK_PRIORITY, NULL );
  if (retVal < 0) error = true;
  retVal = xTaskCreate(blinkTask, "blinkTask", BLINK_TASK_STACK_SIZE,
                      NULL, BLINK_TASK_PRIORITY, NULL );
  if (retVal < 0) error = true;

  if (error)
  {
      led_arduino_on();
      while(1);
  }
  // Start the task scheduler
  vTaskStartScheduler();


}

void loop() {}

//RECEIVE LIDAR TASK BEGIN----------------------------------------------------------------------------
//Tarea que recibe las medidas del lidar
static void receiveLidarTask(void *pvParameters)
{
    while(true)
    {
       while (lidarMeasure(&lidar, &measure, startMapping))
       {
            xQueueSendToBack(lidarSPIQueue, &measure, portMAX_DELAY);
       }
       vTaskDelay(1 / portTICK_PERIOD_MS);
    }
}
//RECEIVE LIDAR TASK END------------------------------------------------------------------------------

//SERVO TASK BEGIN----------------------------------------------------------------------------
//Control del servo
static void servoTask(void *pvParameters)
{
    moveServo(90);
    vTaskDelay(600/portTICK_PERIOD_MS);
    moveServo(180);
    vTaskDelay(600/portTICK_PERIOD_MS);
    moveServo(0);

    while(true)
    {
      if(startServo){
        moveServo(unityCommand.value);
        startServo = false;
      }
      vTaskDelay(1/portTICK_PERIOD_MS);
    }
}
//SERVO TASK END------------------------------------------------------------------------------

//SEND SPI TASK BEGIN----------------------------------------------------------------------------
//Tarea para enviar tramas API via SPI hacia Xbee
static void sendSPITask(void *pvParameters)
{
    byte payload[TX_FRAME_PAYLOAD_BYTES];
    Measure_t measureInQueue;
    uint8_t i;
    while(true)
    {
      i=0;
      while (i<MEASURES_PER_FRAME)
      {
        xQueueReceive(lidarSPIQueue, &measureInQueue, portMAX_DELAY);

        if( ( (measureInQueue.lidarAngle>=0 && measureInQueue.lidarAngle<=205) ||
            (measureInQueue.lidarAngle>=335 && measureInQueue.lidarAngle<=360) ) &&
            (measureInQueue.lidarDistance>=150 && measureInQueue.lidarDistance<=12000))
        {
          //Conversion a bytes
          *( (Measure_t *) (payload + (i * MEASURE_BYTES_SIZE) ) ) = measureInQueue;
          i++;
        }
      }
      xSemaphoreTake(SPIMutex, portMAX_DELAY);
        SPIcommunication(payload);
      xSemaphoreGive(SPIMutex);
      vTaskDelay(1 / portTICK_PERIOD_MS);
    }
}
//SEND SPI TASK END------------------------------------------------------------------------------

//RECEIVE SPI TASK BEGIN------------------------------------------------------------------------------
//Tarea que recibe tramas API via SPI desde Xbee
static void receiveSPITask(void *pvParameters)
{
    while (1)
    {
      if (digitalRead(SPI_ATTN)== LOW)
      {
        xSemaphoreTake(SPIMutex, portMAX_DELAY);
          SPIcommunication(&unityCommand);
        xSemaphoreGive(SPIMutex);
        applyCommand();
      }
      vTaskDelay(1 / portTICK_PERIOD_MS);
    }
}
//RECEIVE SPI TASK END------------------------------------------------------------------------------

//MOTOR TASK BEGIN----------------------------------------------------------------------------
//Control del movimiento del vehiculo
static void motorTask(void *pvParameters)
{
    while(true)
    {
      if(startMotors)
        startMotors = motorsControl(unityCommand.command, unityCommand.value);
      vTaskDelay(10 / portTICK_PERIOD_MS);
    }
}
//MOTOR TASK END------------------------------------------------------------------------------

//BLINK TASK BEGIN----------------------------------------------------------------------------
static void blinkTask(void *pvParameters)
{
    while(true)
    {
      digitalWrite(LED_BLINK, HIGH);
      vTaskDelay(30 / portTICK_PERIOD_MS);
      digitalWrite(LED_BLINK, LOW);
      vTaskDelay(1000 / portTICK_PERIOD_MS);
    }
}
//BLINK TASK END------------------------------------------------------------------------------

//Funcion que interpreta comando de Unity
void applyCommand()
{
  //Comando para activar lidar (empezar mapeo)
  if(unityCommand.command == 'M')
    startMapping = true;
  //Comando para desactivar lidar (detener mapeo)
  if(unityCommand.command == 'P')
    startMapping = false;
  //Comandos para mover vehiculo (adelante, atras, izquierda, derecha)
  if( (unityCommand.command == 'F') || (unityCommand.command == 'B') ||
      (unityCommand.command == 'L') || (unityCommand.command == 'R') )
    startMotors = true;
  //Comando para girar servo
  if(unityCommand.command == 'S')
    startServo = true;
}

/*Función que controla el ancho de pulso
 *@param servoAngle angulo donde se debe ubicar el servo
*/
void moveServo(unsigned int servoAngle){
  unsigned long time;
  int pulseWidth;

  pulseWidth = 900 + servoAngle * 6.666667;

  digitalWrite(SERVO_CONTROL, HIGH);
  time = micros();
  while(micros()-time < pulseWidth){
    __asm__("nop");
  }
  digitalWrite(SERVO_CONTROL, LOW);
}


void pinConfig()
{
  pinMode(LED_BLINK, OUTPUT);

  //Lidar setup
  pinMode(RPLIDAR_MOTOR_CTL, OUTPUT);

  //SPI setup
  pinMode(SPI_SS, OUTPUT);
  pinMode(SPI_ATTN, INPUT);
  digitalWrite(SPI_SS, HIGH);

  //Motor setup
  pinMode(LEFT_MOTOR_DIRECTION, OUTPUT);
  pinMode(RIGHT_MOTOR_DIRECTION, OUTPUT);
  pinMode(LEFT_MOTOR_SPEED, OUTPUT);
  pinMode(RIGHT_MOTOR_SPEED, OUTPUT);
  pinMode(ENCODER, INPUT);

  //Servo setup
  pinMode(SERVO_CONTROL, OUTPUT);
}


void led_arduino_on()
{
  digitalWrite(LED_BLINK, HIGH);
}
